<?php

namespace Drp\LdapAuthentication;

use Adldap\Laravel\AdldapServiceProvider;
use Adldap\Laravel\Exceptions\ConfigurationMissingException;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Foundation\Application;
use Adldap\Adldap;

/**
 * Class DrpldapServiceProvider
 * @package Drp\LdapAuthentication
 */
class DrpldapServiceProvider extends AdldapServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     * @var boolean
     */
    protected $defer = false;

    /**
     * Run service provider boot operations.
     * @return void
     */
    public function boot()
    {
        $this->bootLdap();
        $this->bootAuth();
    }

    /**
     * Register the service provider.
     * @return void
     */
    public function register()
    {
        // Bind the Adldap instance to the IoC
        $this->app->bind('adldap', function (Application $app) {
            $config = $app->make('config');

            $settings = $config->get('adldap');

            // Verify configuration.
            if (is_null($settings)) {
                $message = 'Adldap configuration could not be found.' .
                ' Try re-publishing using `php artisan vendor:publish --tag="adldap"`.';

                throw new ConfigurationMissingException($message);
            }

            // Create a new Adldap instance.
            $ad = new Adldap(
                $settings['connection_settings'],
                new $settings['connection'](),
                $settings['auto_connect']
            );

            return $ad;
        });

        // Bind the Adldap contract to the Adldap object
        // in the IoC for dependency injection.
        $this->app->bind('Adldap\Contracts\Adldap', 'adldap');
    }

    /**
     * @return void
     */
    protected function bootAuth()
    {
        $auth = base_path('vendor/adldap2/adldap2-laravel/src/Config/auth.php');

        // Add publishable configuration.
        $this->publishes([
            $auth => config_path('adldap_auth.php'),
        ], 'adldap');

        // Register the adldap auth user provider.
        Auth::provider('ldap', function ($app, array $config) {
            return new DrpldapAuthUserProvider(
                $app['hash'],
                $config['model'],
                app(GroupFinder::class)
            );
        });

    }

    /**
     * @return void
     */
    protected function bootLdap()
    {
        $this->publishes([
            __DIR__ . '/../database/migrations/' => database_path('migrations')
        ], 'migrations');

        $config = __DIR__ . '/../config/config.php';

        $this->publishes([
            $config => config_path('adldap.php'),
        ], 'adldap');

    }

    /**
     * Get the services provided by the provider.
     * @return array
     */
    public function provides()
    {
        return ['auth'];
    }
}
